@@ -12,9 +12,9 @@
  * for more details.
  */
 
+#include "shareusergroupwidget.h"
 #include "ui_shareusergroupwidget.h"
 #include "ui_shareuserline.h"
-#include "shareusergroupwidget.h"
 #include "account.h"
 #include "folderman.h"
 #include "folder.h"
@@ -26,7 +26,7 @@
 #include "thumbnailjob.h"
 #include "sharee.h"
 #include "sharemanager.h"
-#include "theme.h"
+#include "guiutility.h"
 
 #include "QProgressIndicator.h"
 #include <QBuffer>
@@ -35,6 +35,7 @@
 #include <QFileInfo>
 #include <QAbstractProxyModel>
 #include <QCompleter>
+#include <qscrollarea.h>
 #include <qlayout.h>
 #include <QPropertyAnimation>
 #include <QMenu>
@@ -44,7 +45,6 @@
 #include <QCryptographicHash>
 #include <QColor>
 #include <QPainter>
-#include <QListWidget>
 
 namespace OCC {
 
@@ -88,33 +88,27 @@ ShareUserGroupWidget::ShareUserGroupWidget(AccountPtr account,
     connect(_manager, &ShareManager::shareCreated, this, &ShareUserGroupWidget::getShares);
     connect(_manager, &ShareManager::serverError, this, &ShareUserGroupWidget::displayError);
     connect(_ui->shareeLineEdit, &QLineEdit::returnPressed, this, &ShareUserGroupWidget::slotLineEditReturn);
-    connect(_ui->confirmShare, &QAbstractButton::clicked, this, &ShareUserGroupWidget::slotLineEditReturn);
-    //TODO connect(_ui->privateLinkText, &QLabel::linkActivated, this, &ShareUserGroupWidget::slotPrivateLinkShare);
+    connect(_ui->privateLinkText, &QLabel::linkActivated, this, &ShareUserGroupWidget::slotPrivateLinkShare);
 
     // By making the next two QueuedConnections we can override
     // the strings the completer sets on the line edit.
-    connect(_completer, SIGNAL(activated(QModelIndex)), SLOT(slotCompleterActivated(QModelIndex)),
+    connect(_completer, qOverload<const QModelIndex &>(&QCompleter::activated), this, &ShareUserGroupWidget::slotCompleterActivated,
         Qt::QueuedConnection);
-    connect(_completer, SIGNAL(highlighted(QModelIndex)), SLOT(slotCompleterHighlighted(QModelIndex)),
+    connect(_completer, qOverload<const QModelIndex &>(&QCompleter::highlighted), this, &ShareUserGroupWidget::slotCompleterHighlighted,
         Qt::QueuedConnection);
 
     // Queued connection so this signal is recieved after textChanged
     connect(_ui->shareeLineEdit, &QLineEdit::textEdited,
         this, &ShareUserGroupWidget::slotLineEditTextEdited, Qt::QueuedConnection);
-    _ui->shareeLineEdit->installEventFilter(this);
     connect(&_completionTimer, &QTimer::timeout, this, &ShareUserGroupWidget::searchForSharees);
     _completionTimer.setSingleShot(true);
     _completionTimer.setInterval(600);
 
+    setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Expanding);
     _ui->errorLabel->hide();
 
-    // TODO Progress Indicator where should it go?
     // Setup the sharee search progress indicator
-    //_ui->shareeHorizontalLayout->addWidget(&_pi_sharee);
-
-    _parentScrollArea = parentWidget()->findChild<QScrollArea*>("scrollArea");
-
-    customizeStyle();
+    _ui->shareeHorizontalLayout->addWidget(&_pi_sharee);
 }
 
 ShareUserGroupWidget::~ShareUserGroupWidget()
@@ -125,7 +119,6 @@ ShareUserGroupWidget::~ShareUserGroupWidget()
 void ShareUserGroupWidget::on_shareeLineEdit_textChanged(const QString &)
 {
     _completionTimer.stop();
-    emit togglePublicLinkShare(false);
 }
 
 void ShareUserGroupWidget::slotLineEditTextEdited(const QString &text)
@@ -135,7 +128,6 @@ void ShareUserGroupWidget::slotLineEditTextEdited(const QString &text)
     // Then we restart the timer here if the user touched a key
     if (!text.isEmpty()) {
         _completionTimer.start();
-        emit togglePublicLinkShare(true);
     }
 }
 
@@ -172,7 +164,8 @@ void ShareUserGroupWidget::searchForSharees()
     QSharedPointer<Sharee> currentUser(new Sharee(_account->credentials()->user(), "", Sharee::Type::User));
     blacklist << currentUser;
 
-    foreach (auto sw, _parentScrollArea->findChildren<ShareUserLine *>()) {
+    const auto &shareUserLines = _ui->scrollArea->findChildren<ShareUserLine *>();
+    for (auto *sw : shareUserLines) {
         blacklist << sw->share()->getShareWith();
     }
     _ui->errorLabel->hide();
@@ -186,61 +179,44 @@ void ShareUserGroupWidget::getShares()
 
 void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>> &shares)
 {
-    QScrollArea *scrollArea = _parentScrollArea;
+    QScrollArea *scrollArea = _ui->scrollArea;
 
     auto newViewPort = new QWidget(scrollArea);
     auto layout = new QVBoxLayout(newViewPort);
-    layout->setContentsMargins(0, 0, 0, 0);
-    int x = 0;
-    int height = 0;
-    QList<QString> linkOwners({});
+    layout->setMargin(0);
+    layout->setSpacing(0);
 
-    foreach (const auto &share, shares) {
-        // We don't handle link shares, only TypeUser or TypeGroup
+    QSize minimumSize = newViewPort->sizeHint();
+    int x = 0;
+
+    for (const auto &share : shares) {
+        // We don't handle link shares
         if (share->getShareType() == Share::TypeLink) {
-            if(!share->getUidOwner().isEmpty() &&
-                    share->getUidOwner() != share->account()->davUser()){
-                linkOwners.append(share->getOwnerDisplayName());
-             }
             continue;
         }
 
-        // the owner of the file that shared it first
-        // leave out if it's the current user
-        if(x == 0 && !share->getUidOwner().isEmpty() && !(share->getUidOwner() == _account->credentials()->user())) {
-            _ui->mainOwnerLabel->setText(QString("Shared with you by ").append(share->getOwnerDisplayName()));
-        }
-
-        ShareUserLine *s = new ShareUserLine(share, _maxSharingPermissions, _isFile, _parentScrollArea);
+        ShareUserLine *s = new ShareUserLine(share, _maxSharingPermissions, _isFile, _ui->scrollArea);
         connect(s, &ShareUserLine::resizeRequested, this, &ShareUserGroupWidget::slotAdjustScrollWidgetSize);
         connect(s, &ShareUserLine::visualDeletionDone, this, &ShareUserGroupWidget::getShares);
         s->setBackgroundRole(layout->count() % 2 == 0 ? QPalette::Base : QPalette::AlternateBase);
-
-        // Connect styleChanged events to our widget, so it can adapt (Dark-/Light-Mode switching)
-        connect(this, &ShareUserGroupWidget::styleChanged, s, &ShareUserLine::slotStyleChanged);
-
         layout->addWidget(s);
 
         x++;
         if (x <= 3) {
-            height = newViewPort->sizeHint().height();
+            minimumSize = newViewPort->sizeHint();
+        } else {
+            minimumSize.rwidth() = qMax(newViewPort->sizeHint().width(), minimumSize.width());
         }
     }
-
-    foreach (const QString &owner, linkOwners) {
-        auto ownerLabel = new QLabel(QString(owner + " shared via link"));
-        layout->addWidget(ownerLabel);
-        ownerLabel->setVisible(true);
-
-        x++;
-        if (x <= 6) {
-            height = newViewPort->sizeHint().height();
-        }
+    if (layout->isEmpty()) {
+        layout->addWidget(new QLabel(tr("The item is not shared with any users or groups")));
+    } else {
+        layout->addStretch(1);
     }
 
-    scrollArea->setFrameShape(x > 6 ? QFrame::StyledPanel : QFrame::NoFrame);
-    scrollArea->setVisible(!shares.isEmpty());
-    scrollArea->setFixedHeight(height);
+    minimumSize.rwidth() += layout->spacing();
+    minimumSize.rheight() += layout->spacing();
+    scrollArea->setMinimumSize(minimumSize);
     scrollArea->setWidget(newViewPort);
 
     _disableCompleterActivated = false;
@@ -249,13 +225,14 @@ void ShareUserGroupWidget::slotSharesFetched(const QList<QSharedPointer<Share>>
 
 void ShareUserGroupWidget::slotAdjustScrollWidgetSize()
 {
-    QScrollArea *scrollArea = _parentScrollArea;
-    int count = scrollArea->findChildren<ShareUserLine *>().count();
-    scrollArea->setVisible(count > 0);
-    if (count > 0 && count <= 3) {
-        scrollArea->setFixedHeight(scrollArea->widget()->sizeHint().height());
+    QScrollArea *scrollArea = _ui->scrollArea;
+    if (scrollArea->findChildren<ShareUserLine *>().count() <= 3) {
+        auto minimumSize = scrollArea->widget()->sizeHint();
+        auto spacing = scrollArea->widget()->layout()->spacing();
+        minimumSize.rwidth() += spacing;
+        minimumSize.rheight() += spacing;
+        scrollArea->setMinimumSize(minimumSize);
     }
-    scrollArea->setFrameShape(count > 3 ? QFrame::StyledPanel : QFrame::NoFrame);
 }
 
 void ShareUserGroupWidget::slotPrivateLinkShare()
@@ -263,10 +240,12 @@ void ShareUserGroupWidget::slotPrivateLinkShare()
     auto menu = new QMenu(this);
     menu->setAttribute(Qt::WA_DeleteOnClose);
 
-    // this icon is not handled by slotStyleChanged() -> customizeStyle but we can live with that
-    menu->addAction(Theme::createColorAwareIcon(":/client/resources/copy.svg"),
-                    tr("Copy link"),
-        this, SLOT(slotPrivateLinkCopy()));
+    menu->addAction(tr("Open link in browser"),
+        this, &ShareUserGroupWidget::slotPrivateLinkOpenBrowser);
+    menu->addAction(tr("Copy link to clipboard"),
+        this, &ShareUserGroupWidget::slotPrivateLinkCopy);
+    menu->addAction(tr("Send link by email"),
+        this, &ShareUserGroupWidget::slotPrivateLinkEmail);
 
     menu->exec(QCursor::pos());
 }
@@ -292,36 +271,38 @@ void ShareUserGroupWidget::slotCompleterActivated(const QModelIndex &index)
         return;
     }
 
-// TODO Progress Indicator where should it go?
-//    auto indicator = new QProgressIndicator(viewPort);
-//    indicator->startAnimation();
-//    if (layout->count() == 1) {
-//        // No shares yet! Remove the label, add some stretch.
-//        delete layout->itemAt(0)->widget();
-//        layout->addStretch(1);
-//    }
-//    layout->insertWidget(layout->count() - 1, indicator);
+    /*
+     * Add spinner to the bottom of the widget list
+     */
+    auto viewPort = _ui->scrollArea->widget();
+    auto layout = qobject_cast<QVBoxLayout *>(viewPort->layout());
+    auto indicator = new QProgressIndicator(viewPort);
+    indicator->startAnimation();
+    if (layout->count() == 1) {
+        // No shares yet! Remove the label, add some stretch.
+        delete layout->itemAt(0)->widget();
+        layout->addStretch(1);
+    }
+    layout->insertWidget(layout->count() - 1, indicator);
 
     /*
      * Don't send the reshare permissions for federated shares for servers <9.1
      * https://github.com/owncloud/core/issues/22122#issuecomment-185637344
      * https://github.com/owncloud/client/issues/4996
      */
+    const SharePermissions defaultPermissions = static_cast<SharePermissions>(_account->capabilities().defaultPermissions());
     if (sharee->type() == Sharee::Federated
         && _account->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        int permissions = SharePermissionRead | SharePermissionUpdate;
+        SharePermissions permissions = SharePermissionRead | SharePermissionUpdate;
         if (!_isFile) {
             permissions |= SharePermissionCreate | SharePermissionDelete;
         }
+        permissions &= defaultPermissions;
         _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), SharePermission(permissions));
+            sharee->shareWith(), permissions);
     } else {
-
-        // Default permissions on creation
-        int permissions = SharePermissionCreate | SharePermissionUpdate
-                | SharePermissionDelete | SharePermissionShare;
         _manager->createShare(_sharePath, Share::ShareType(sharee->type()),
-            sharee->shareWith(), SharePermission(permissions));
+            sharee->shareWith(), _maxSharingPermissions & defaultPermissions);
     }
 
     _ui->shareeLineEdit->setEnabled(false);
@@ -340,9 +321,7 @@ void ShareUserGroupWidget::displayError(int code, const QString &message)
     _pi_sharee.stopAnimation();
 
     // Also remove the spinner in the widget list, if any
-    foreach (auto pi, _parentScrollArea->findChildren<QProgressIndicator *>()) {
-        delete pi;
-    }
+    qDeleteAll(_ui->scrollArea->findChildren<QProgressIndicator *>());
 
     qCWarning(lcSharing) << "Sharing error from server" << code << message;
     _ui->errorLabel->setText(message);
@@ -368,25 +347,6 @@ void ShareUserGroupWidget::slotPrivateLinkEmail()
         this);
 }
 
-void ShareUserGroupWidget::slotStyleChanged()
-{
-    customizeStyle();
-
-    // Notify the other widgets (ShareUserLine in this case, Dark-/Light-Mode switching)
-    emit styleChanged();
-}
-
-void ShareUserGroupWidget::customizeStyle()
-{
-    _ui->confirmShare->setIcon(Theme::createColorAwareIcon(":/client/resources/confirm.svg"));
-
-    _pi_sharee.setColor(QGuiApplication::palette().color(QPalette::Text));
-
-    foreach (auto pi, _parentScrollArea->findChildren<QProgressIndicator *>()) {
-        pi->setColor(QGuiApplication::palette().color(QPalette::Text));;
-    }
-}
-
 ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
     SharePermissions maxSharingPermissions,
     bool isFile,
@@ -398,91 +358,72 @@ ShareUserLine::ShareUserLine(QSharedPointer<Share> share,
 {
     _ui->setupUi(this);
 
-    _ui->sharedWith->setElideMode(Qt::ElideRight);
     _ui->sharedWith->setText(share->getShareWith()->format());
 
-    // adds permissions
-    // can edit permission
-    bool enabled = (maxSharingPermissions & SharePermissionUpdate);
-    if(!_isFile) enabled = enabled && (maxSharingPermissions & SharePermissionCreate &&
-                                      maxSharingPermissions & SharePermissionDelete);
-    _ui->permissionsEdit->setEnabled(enabled);
-    connect(_ui->permissionsEdit, &QAbstractButton::clicked, this, &ShareUserLine::slotEditPermissionsChanged);
-
-    // create menu with checkable permissions
+    // Create detailed permissions menu
     QMenu *menu = new QMenu(this);
-    _permissionReshare= new QAction(tr("Can reshare"), this);
-    _permissionReshare->setCheckable(true);
-    _permissionReshare->setEnabled(maxSharingPermissions & SharePermissionShare);
-    menu->addAction(_permissionReshare);
-    connect(_permissionReshare, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
-
-    menu->addSeparator();
-
-      // Adds action to delete share widget
-      QIcon deleteicon = QIcon::fromTheme(QLatin1String("user-trash"),QIcon(QLatin1String(":/client/resources/delete.png")));
-      _deleteShareButton= new QAction(deleteicon,tr("Unshare"), this);
-
-    menu->addAction(_deleteShareButton);
-    connect(_deleteShareButton, &QAction::triggered, this, &ShareUserLine::on_deleteShareButton_clicked);
+    _permissionCreate = new QAction(tr("create"), this);
+    _permissionCreate->setCheckable(true);
+    _permissionCreate->setEnabled(maxSharingPermissions & SharePermissionCreate);
+    _permissionUpdate = new QAction(tr("change"), this);
+    _permissionUpdate->setCheckable(true);
+    _permissionUpdate->setEnabled(maxSharingPermissions & SharePermissionUpdate);
+    _permissionDelete = new QAction(tr("delete"), this);
+    _permissionDelete->setCheckable(true);
+    _permissionDelete->setEnabled(maxSharingPermissions & SharePermissionDelete);
 
+    menu->addAction(_permissionUpdate);
     /*
      * Files can't have create or delete permissions
      */
     if (!_isFile) {
-        _permissionCreate = new QAction(tr("Can create"), this);
-        _permissionCreate->setCheckable(true);
-        _permissionCreate->setEnabled(maxSharingPermissions & SharePermissionCreate);
         menu->addAction(_permissionCreate);
-        connect(_permissionCreate, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
-
-        _permissionChange = new QAction(tr("Can change"), this);
-        _permissionChange->setCheckable(true);
-        _permissionChange->setEnabled(maxSharingPermissions & SharePermissionUpdate);
-        menu->addAction(_permissionChange);
-        connect(_permissionChange, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
-
-        _permissionDelete = new QAction(tr("Can delete"), this);
-        _permissionDelete->setCheckable(true);
-        _permissionDelete->setEnabled(maxSharingPermissions & SharePermissionDelete);
         menu->addAction(_permissionDelete);
-        connect(_permissionDelete, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
     }
-
     _ui->permissionToolButton->setMenu(menu);
     _ui->permissionToolButton->setPopupMode(QToolButton::InstantPopup);
+    _ui->permissionToolButton->setIcon(Utility::getCoreIcon(QStringLiteral("more")));
 
-    // icon now set in: customizeStyle
-    /*QIcon icon(QLatin1String(":/client/resources/more.svg"));
-    _ui->permissionToolButton->setIcon(icon);*/
+    // If there's only a single entry in the detailed permission menu, hide it
+    if (menu->actions().size() == 1) {
+        _ui->permissionToolButton->hide();
+    }
 
     // Set the permissions checkboxes
     displayPermissions();
 
+    _ui->permissionShare->setEnabled(maxSharingPermissions & SharePermissionShare);
+    _ui->permissionsEdit->setEnabled(maxSharingPermissions
+        & (SharePermissionCreate | SharePermissionUpdate | SharePermissionDelete));
+
+    connect(_permissionUpdate, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
+    connect(_permissionCreate, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
+    connect(_permissionDelete, &QAction::triggered, this, &ShareUserLine::slotPermissionsChanged);
+    connect(_ui->permissionShare, &QAbstractButton::clicked, this, &ShareUserLine::slotPermissionsChanged);
+    connect(_ui->permissionsEdit, &QAbstractButton::clicked, this, &ShareUserLine::slotEditPermissionsChanged);
+
     /*
-     * We don't show permission share for federated shares with server <9.1
+     * We don't show permssion share for federated shares with server <9.1
      * https://github.com/owncloud/core/issues/22122#issuecomment-185637344
      * https://github.com/owncloud/client/issues/4996
      */
     if (share->getShareType() == Share::TypeRemote
         && share->account()->serverVersionInt() < Account::makeServerVersion(9, 1, 0)) {
-        _permissionReshare->setVisible(false);
+        _ui->permissionShare->setVisible(false);
         _ui->permissionToolButton->setVisible(false);
     }
 
     connect(share.data(), &Share::permissionsSet, this, &ShareUserLine::slotPermissionsSet);
     connect(share.data(), &Share::shareDeleted, this, &ShareUserLine::slotShareDeleted);
 
-    // _ui->deleteShareButton->setIcon(QIcon::fromTheme(QLatin1String("user-trash"),
-    //                                                  QIcon(QLatin1String(":/client/resources/delete.png"))));
+    _ui->deleteShareButton->setIcon(QIcon::fromTheme(QStringLiteral("user-trash"),
+        Utility::getCoreIcon(QStringLiteral("delete"))));
 
     if (!share->account()->capabilities().shareResharing()) {
-        _permissionReshare->setVisible(false);
+        _ui->permissionShare->hide();
     }
 
     loadAvatar();
-
-    customizeStyle();
 }
 
 void ShareUserLine::loadAvatar()
@@ -518,15 +459,26 @@ void ShareUserLine::loadAvatar()
     // The avatar label is the first character of the user name.
     const QString text = _share->getShareWith()->displayName();
     _ui->avatar->setText(text.at(0).toUpper());
+
+    /* Start the network job to fetch the avatar data.
+     *
+     * Currently only regular users can have avatars.
+     */
+    return;
+    if (_share->getShareWith()->type() == Sharee::User) {
+        AvatarJob *job = new AvatarJob(_share->account(), _share->getShareWith()->shareWith(), avatarSize, this);
+        connect(job, &AvatarJob::avatarPixmap, this, &ShareUserLine::slotAvatarLoaded);
+        job->start();
+    }
 }
 
-void ShareUserLine::slotAvatarLoaded(QImage avatar)
+void ShareUserLine::slotAvatarLoaded(const QPixmap &avatar)
 {
     if (avatar.isNull())
         return;
 
-    avatar = AvatarJob::makeCircularAvatar(avatar);
-    _ui->avatar->setPixmap(QPixmap::fromImage(avatar));
+    const QPixmap _avatar = AvatarJob::makeCircularAvatar(avatar);
+    _ui->avatar->setPixmap(_avatar);
 
     // Remove the stylesheet for the fallback avatar
     _ui->avatar->setStyleSheet("");
@@ -556,28 +508,25 @@ void ShareUserLine::slotEditPermissionsChanged()
 
     Share::Permissions permissions = SharePermissionRead;
 
-    //  folders edit = CREATE, READ, UPDATE, DELETE
-    //  files edit = READ + UPDATE
+    if (_ui->permissionShare->checkState() == Qt::Checked) {
+        permissions |= SharePermissionShare;
+    }
+
     if (_ui->permissionsEdit->checkState() == Qt::Checked) {
+        if (_permissionUpdate->isEnabled())
+            permissions |= SharePermissionUpdate;
 
         /*
          * Files can't have create or delete permisisons
          */
         if (!_isFile) {
-            if (_permissionChange->isEnabled())
-                permissions |= SharePermissionUpdate;
             if (_permissionCreate->isEnabled())
                 permissions |= SharePermissionCreate;
             if (_permissionDelete->isEnabled())
                 permissions |= SharePermissionDelete;
-        } else {
-            permissions |= SharePermissionUpdate;
         }
     }
 
-    if(_isFile && _permissionReshare->isEnabled() && _permissionReshare->isChecked())
-        permissions |= SharePermissionShare;
-
     _share->setPermissions(permissions);
 }
 
@@ -587,19 +536,20 @@ void ShareUserLine::slotPermissionsChanged()
 
     Share::Permissions permissions = SharePermissionRead;
 
-    if (_permissionReshare->isChecked())
-        permissions |= SharePermissionShare;
+    if (_permissionUpdate->isChecked()) {
+        permissions |= SharePermissionUpdate;
+    }
 
-    if (!_isFile) {
-        if (_permissionChange->isChecked())
-            permissions |= SharePermissionUpdate;
-        if (_permissionCreate->isChecked())
-            permissions |= SharePermissionCreate;
-        if (_permissionDelete->isChecked())
-            permissions |= SharePermissionDelete;
-    } else {
-        if (_ui->permissionsEdit->isChecked())
-            permissions |= SharePermissionUpdate;
+    if (_permissionCreate->isChecked()) {
+        permissions |= SharePermissionCreate;
+    }
+
+    if (_permissionDelete->isChecked()) {
+        permissions |= SharePermissionDelete;
+    }
+
+    if (_ui->permissionShare->checkState() == Qt::Checked) {
+        permissions |= SharePermissionShare;
     }
 
     _share->setPermissions(permissions);
@@ -614,7 +564,7 @@ void ShareUserLine::slotDeleteAnimationFinished()
     // There is a painting bug where a small line of this widget isn't
     // properly cleared. This explicit repaint() call makes sure any trace of
     // the share widget is removed once it's destroyed. #4189
-    connect(this, SIGNAL(destroyed(QObject *)), parentWidget(), SLOT(repaint()));
+    connect(this, &ShareUserLine::destroyed, parentWidget(), qOverload<>(&QWidget::repaint));
 }
 
 void ShareUserLine::slotShareDeleted()
@@ -644,41 +594,23 @@ QSharedPointer<Share> ShareUserLine::share() const
 
 void ShareUserLine::displayPermissions()
 {
-    auto perm = _share->getPermissions();
+    const SharePermissions perm = _share->getPermissions();
 
-//  folders edit = CREATE, READ, UPDATE, DELETE
-//  files edit = READ + UPDATE
-    if (perm & SharePermissionUpdate && (_isFile ||
-                                         (perm & SharePermissionCreate && perm & SharePermissionDelete))) {
+    _permissionUpdate->setChecked(perm & SharePermissionUpdate);
+    _permissionCreate->setChecked(!_isFile && perm & SharePermissionCreate);
+    _permissionDelete->setChecked(!_isFile && perm & SharePermissionDelete);
+
+    if (perm & SharePermissionUpdate
+        && (_isFile
+            || (perm & SharePermissionCreate
+                && perm & SharePermissionDelete))) {
         _ui->permissionsEdit->setCheckState(Qt::Checked);
-    } else if (!_isFile && perm & (SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete)) {
+    } else if (perm & (SharePermissionUpdate | SharePermissionCreate | SharePermissionDelete)) {
         _ui->permissionsEdit->setCheckState(Qt::PartiallyChecked);
-    } else if(perm & SharePermissionRead) {
+    } else {
         _ui->permissionsEdit->setCheckState(Qt::Unchecked);
     }
 
-//  edit is independent of reshare
-    if (perm & SharePermissionShare)
-        _permissionReshare->setChecked(true);
-
-    if(!_isFile){
-        _permissionCreate->setChecked(perm & SharePermissionCreate);
-        _permissionChange->setChecked(perm & SharePermissionUpdate);
-        _permissionDelete->setChecked(perm & SharePermissionDelete);
-    }
+    _ui->permissionShare->setChecked(perm & SharePermissionShare);
 }
-
-void ShareUserLine::slotStyleChanged()
-{
-    customizeStyle();
-}
-
-void ShareUserLine::customizeStyle()
-{
-    _ui->permissionToolButton->setIcon(Theme::createColorAwareIcon(":/client/resources/more.svg"));
-
-    QIcon deleteicon = QIcon::fromTheme(QLatin1String("user-trash"),Theme::createColorAwareIcon(QLatin1String(":/client/resources/delete.png")));
-    _deleteShareButton->setIcon(deleteicon);
-}
-
 }
