@@ -14,7 +14,6 @@
 
 #include "config.h"
 #include "propagateupload.h"
-#include "propagateuploadencrypted.h"
 #include "owncloudpropagator_p.h"
 #include "networkjobs.h"
 #include "account.h"
@@ -27,25 +26,21 @@
 #include "syncengine.h"
 #include "propagateremotedelete.h"
 #include "common/asserts.h"
-#include "networkjobs.h"
-#include "clientsideencryption.h"
-#include "clientsideencryptionjobs.h"
 
 #include <QNetworkAccessManager>
 #include <QFileInfo>
 #include <QDir>
 #include <QJsonDocument>
 #include <QJsonObject>
-#include <QFileInfo>
-
 #include <cmath>
 #include <cstring>
 
 namespace OCC {
 
-Q_LOGGING_CATEGORY(lcPutJob, "nextcloud.sync.networkjob.put", QtInfoMsg)
-Q_LOGGING_CATEGORY(lcPollJob, "nextcloud.sync.networkjob.poll", QtInfoMsg)
-Q_LOGGING_CATEGORY(lcPropagateUpload, "nextcloud.sync.propagator.upload", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPutJob, "sync.networkjob.put", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateUpload, "sync.propagator.upload", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateUploadV1, "sync.propagator.upload.v1", QtInfoMsg)
+Q_LOGGING_CATEGORY(lcPropagateUploadNG, "sync.propagator.upload.ng", QtInfoMsg)
 
 /**
  * We do not want to upload files that are currently being modified.
@@ -61,7 +56,7 @@ static bool fileIsStillChanging(const SyncFileItem &item)
     const QDateTime modtime = Utility::qDateTimeFromTime_t(item._modtime);
     const qint64 msSinceMod = modtime.msecsTo(QDateTime::currentDateTimeUtc());
 
-    return msSinceMod < SyncEngine::minimumFileAgeForUpload
+    return std::chrono::milliseconds(msSinceMod) < SyncEngine::minimumFileAgeForUpload
         // if the mtime is too much in the future we *do* upload the file
         && msSinceMod > -10000;
 }
@@ -75,7 +70,7 @@ PUTFileJob::~PUTFileJob()
 void PUTFileJob::start()
 {
     QNetworkRequest req;
-    for (QMap<QByteArray, QByteArray>::const_iterator it = _headers.begin(); it != _headers.end(); ++it) {
+    for (auto it = _headers.cbegin(); it != _headers.cend(); ++it) {
         req.setRawHeader(it.key(), it.value());
     }
 
@@ -87,78 +82,20 @@ void PUTFileJob::start()
         sendRequest("PUT", makeDavUrl(path()), req, _device);
     }
 
-    if (reply()->error() != QNetworkReply::NoError) {
-        qCWarning(lcPutJob) << " Network error: " << reply()->errorString();
-    }
-
     connect(reply(), &QNetworkReply::uploadProgress, this, &PUTFileJob::uploadProgress);
     connect(this, &AbstractNetworkJob::networkActivity, account().data(), &Account::propagatorNetworkActivity);
     _requestTimer.start();
     AbstractNetworkJob::start();
 }
 
-void PollJob::start()
+bool PUTFileJob::finished()
 {
-    setTimeout(120 * 1000);
-    QUrl accountUrl = account()->url();
-    QUrl finalUrl = QUrl::fromUserInput(accountUrl.scheme() + QLatin1String("://") + accountUrl.authority()
-        + (path().startsWith('/') ? QLatin1String("") : QLatin1String("/")) + path());
-    sendRequest("GET", finalUrl);
-    connect(reply(), &QNetworkReply::downloadProgress, this, &AbstractNetworkJob::resetTimeout);
-    AbstractNetworkJob::start();
-}
+    _device->close();
 
-bool PollJob::finished()
-{
-    QNetworkReply::NetworkError err = reply()->error();
-    if (err != QNetworkReply::NoError) {
-        _item->_httpErrorCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        _item->_status = classifyError(err, _item->_httpErrorCode);
-        _item->_errorString = errorString();
-
-        if (_item->_status == SyncFileItem::FatalError || _item->_httpErrorCode >= 400) {
-            if (_item->_status != SyncFileItem::FatalError
-                && _item->_httpErrorCode != 503) {
-                SyncJournalDb::PollInfo info;
-                info._file = _item->_file;
-                // no info._url removes it from the database
-                _journal->setPollInfo(info);
-                _journal->commit("remove poll info");
-            }
-            emit finishedSignal();
-            return true;
-        }
-        start();
-        return false;
-    }
-
-    QByteArray jsonData = reply()->readAll().trimmed();
-    qCInfo(lcPollJob) << ">" << jsonData << "<" << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-    QJsonParseError jsonParseError;
-    QJsonObject status = QJsonDocument::fromJson(jsonData, &jsonParseError).object();
-    if (jsonParseError.error != QJsonParseError::NoError) {
-        _item->_errorString = tr("Invalid JSON reply from the poll URL");
-        _item->_status = SyncFileItem::NormalError;
-        emit finishedSignal();
-        return true;
-    }
-
-    if (status["unfinished"].toBool()) {
-        start();
-        return false;
-    }
-
-    _item->_errorString = status["error"].toString();
-    _item->_status = _item->_errorString.isEmpty() ? SyncFileItem::Success : SyncFileItem::NormalError;
-    _item->_fileId = status["fileid"].toString().toUtf8();
-    _item->_etag = status["etag"].toString().toUtf8();
-    _item->_responseTimeStamp = responseTimestamp();
-
-    SyncJournalDb::PollInfo info;
-    info._file = _item->_file;
-    // no info._url removes it from the database
-    _journal->setPollInfo(info);
-    _journal->commit("remove poll info");
+    qCInfo(lcPutJob) << "PUT of" << reply()->request().url().toString() << "FINISHED WITH STATUS"
+                     << replyStatusString()
+                     << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt()
+                     << reply()->attribute(QNetworkRequest::HttpReasonPhraseAttribute);
 
     emit finishedSignal();
     return true;
@@ -169,73 +106,38 @@ void PropagateUploadFileCommon::setDeleteExisting(bool enabled)
     _deleteExisting = enabled;
 }
 
+
 void PropagateUploadFileCommon::start()
 {
-    if (propagator()->account()->capabilities().clientSideEncryptionAvaliable()) {
-      _uploadEncryptedHelper = new PropagateUploadEncrypted(propagator(), _item);
-      connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::folerNotEncrypted,
-        this, &PropagateUploadFileCommon::setupUnencryptedFile);
-      connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::finalized,
-        this, &PropagateUploadFileCommon::setupEncryptedFile);
-      connect(_uploadEncryptedHelper, &PropagateUploadEncrypted::error,
-        []{ qCDebug(lcPropagateUpload) << "Error setting up encryption."; });
-      _uploadEncryptedHelper->start();
-   } else {
-      setupUnencryptedFile();
-    }
-}
-
-void PropagateUploadFileCommon::setupEncryptedFile(const QString& path, const QString& filename, quint64 size)
-{
-    qCDebug(lcPropagateUpload) << "Starting to upload encrypted file" << path << filename << size;
-    _uploadingEncrypted = true;
-    _fileToUpload._path = path;
-    _fileToUpload._file = filename;
-    _fileToUpload._size = size;
-    startUploadFile();
-}
-
-void PropagateUploadFileCommon::setupUnencryptedFile()
-{
-    _uploadingEncrypted = false;
-    _fileToUpload._file = _item->_file;
-    _fileToUpload._size = _item->_size;
-    _fileToUpload._path = propagator()->getFilePath(_fileToUpload._file);
-    startUploadFile();
-}
-
-void PropagateUploadFileCommon::startUploadFile() {
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0)) {
+    if (propagator()->_abortRequested) {
         return;
     }
 
     // Check if the specific file can be accessed
-    if (propagator()->hasCaseClashAccessibilityProblem(_fileToUpload._file)) {
+    if (propagator()->hasCaseClashAccessibilityProblem(_item->_file)) {
         done(SyncFileItem::NormalError, tr("File %1 cannot be uploaded because another file with the same name, differing only in case, exists").arg(QDir::toNativeSeparators(_item->_file)));
         return;
     }
 
     // Check if we believe that the upload will fail due to remote quota limits
-    const quint64 quotaGuess = propagator()->_folderQuota.value(
-        QFileInfo(_fileToUpload._file).path(), std::numeric_limits<quint64>::max());
-    if (_fileToUpload._size > quotaGuess) {
+    const qint64 quotaGuess = propagator()->_folderQuota.value(
+        QFileInfo(_item->_file).path(), std::numeric_limits<qint64>::max());
+    if (_item->_size > quotaGuess) {
         // Necessary for blacklisting logic
         _item->_httpErrorCode = 507;
         emit propagator()->insufficientRemoteStorage();
-        done(SyncFileItem::DetailError, tr("Upload of %1 exceeds the quota for the folder").arg(Utility::octetsToString(_fileToUpload._size)));
+        done(SyncFileItem::DetailError, tr("Upload of %1 exceeds the quota for the folder").arg(Utility::octetsToString(_item->_size)));
         return;
     }
 
     propagator()->_activeJobList.append(this);
 
     if (!_deleteExisting) {
-        qDebug() << "Running the compute checksum";
         return slotComputeContentChecksum();
     }
 
-    qDebug() << "Deleting the current";
     auto job = new DeleteJob(propagator()->account(),
-        propagator()->_remoteFolder + _fileToUpload._file,
+        propagator()->fullRemotePath(_item->_file),
         this);
     _jobs.append(job);
     connect(job, &DeleteJob::finishedSignal, this, &PropagateUploadFileCommon::slotComputeContentChecksum);
@@ -245,26 +147,19 @@ void PropagateUploadFileCommon::startUploadFile() {
 
 void PropagateUploadFileCommon::slotComputeContentChecksum()
 {
-    qDebug() << "Tryint to compute the checksum of the file";
-    qDebug() << "Still trying to understand if this is the local file or the uploaded one";
-    if (propagator()->_abortRequested.fetchAndAddRelaxed(0)) {
+    if (propagator()->_abortRequested) {
         return;
     }
 
-    const QString filePath = propagator()->getFilePath(_item->_file);
+    const QString filePath = propagator()->fullLocalPath(_item->_file);
 
     // remember the modtime before checksumming to be able to detect a file
-    // change during the checksum calculation - This goes inside of the _item->_file
-    // and not the _fileToUpload because we are checking the original file, not there
-    // probably temporary one.
+    // change during the checksum calculation
     _item->_modtime = FileSystem::getModTime(filePath);
 
-    QByteArray checksumType = contentChecksumType();
+    const QByteArray checksumType = propagator()->account()->capabilities().preferredUploadChecksumType();
 
     // Maybe the discovery already computed the checksum?
-    // Should I compute the checksum of the original (_item->_file)
-    // or the maybe-modified? (_fileToUpload._file) ?
-
     QByteArray existingChecksumType, existingChecksum;
     parseChecksumHeader(_item->_checksumHeader, &existingChecksumType, &existingChecksum);
     if (existingChecksumType == checksumType) {
@@ -307,7 +202,7 @@ void PropagateUploadFileCommon::slotComputeTransmissionChecksum(const QByteArray
         this, &PropagateUploadFileCommon::slotStartUpload);
     connect(computeChecksum, &ComputeChecksum::done,
         computeChecksum, &QObject::deleteLater);
-    const QString filePath = propagator()->getFilePath(_item->_file);
+    const QString filePath = propagator()->fullLocalPath(_item->_file);
     computeChecksum->start(filePath);
 }
 
@@ -324,42 +219,31 @@ void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionCh
         _item->_checksumHeader = _transmissionChecksumHeader;
     }
 
-    const QString fullFilePath = _fileToUpload._path;
-    const QString originalFilePath = propagator()->getFilePath(_item->_file);
+    const QString fullFilePath = propagator()->fullLocalPath(_item->_file);
 
     if (!FileSystem::fileExists(fullFilePath)) {
-      if (_uploadingEncrypted) {
-        _uploadEncryptedHelper->unlockFolder();
-      }
-    done(SyncFileItem::SoftError, tr("File Removed (start upload) %1").arg(fullFilePath));
+        done(SyncFileItem::SoftError, tr("File Removed"));
         return;
     }
     time_t prevModtime = _item->_modtime; // the _item value was set in PropagateUploadFile::start()
     // but a potential checksum calculation could have taken some time during which the file could
     // have been changed again, so better check again here.
 
-    _item->_modtime = FileSystem::getModTime(originalFilePath);
+    _item->_modtime = FileSystem::getModTime(fullFilePath);
     if (prevModtime != _item->_modtime) {
         propagator()->_anotherSyncNeeded = true;
-        if (_uploadingEncrypted) {
-          _uploadEncryptedHelper->unlockFolder();
-        }
-        qDebug() << "prevModtime" << prevModtime << "Curr" << _item->_modtime;
         done(SyncFileItem::SoftError, tr("Local file changed during syncing. It will be resumed."));
         return;
     }
 
-    quint64 fileSize = FileSystem::getSize(fullFilePath);
-    _fileToUpload._size = fileSize;
+    qint64 fileSize = FileSystem::getSize(fullFilePath);
+    _item->_size = fileSize;
 
     // But skip the file if the mtime is too close to 'now'!
     // That usually indicates a file that is still being changed
     // or not yet fully copied to the destination.
     if (fileIsStillChanging(*_item)) {
         propagator()->_anotherSyncNeeded = true;
-        if (_uploadingEncrypted) {
-          _uploadEncryptedHelper->unlockFolder();
-        }
         done(SyncFileItem::SoftError, tr("Local file changed during sync."));
         return;
     }
@@ -367,8 +251,11 @@ void PropagateUploadFileCommon::slotStartUpload(const QByteArray &transmissionCh
     doStartUpload();
 }
 
-UploadDevice::UploadDevice(BandwidthManager *bwm)
-    : _read(0)
+UploadDevice::UploadDevice(const QString &fileName, qint64 start, qint64 size, BandwidthManager *bwm)
+    : _file(fileName)
+    , _start(start)
+    , _size(size)
+    , _read(0)
     , _bandwidthManager(bwm)
     , _bandwidthQuota(0)
     , _readWithProgress(0)
@@ -386,47 +273,50 @@ UploadDevice::~UploadDevice()
     }
 }
 
-bool UploadDevice::prepareAndOpen(const QString &fileName, qint64 start, qint64 size)
+bool UploadDevice::open(QIODevice::OpenMode mode)
 {
-    _data.clear();
-    _read = 0;
+    if (mode & QIODevice::WriteOnly)
+        return false;
+
+    // Get the file size now: _file.fileName() is no longer reliable
+    // on all platforms after openAndSeekFileSharedRead().
+    auto fileDiskSize = FileSystem::getSize(_file.fileName());
 
-    QFile file(fileName);
     QString openError;
-    if (!FileSystem::openAndSeekFileSharedRead(&file, &openError, start)) {
+    if (!FileSystem::openAndSeekFileSharedRead(&_file, &openError, _start)) {
         setErrorString(openError);
         return false;
     }
 
-    size = qBound(0ll, size, FileSystem::getSize(fileName) - start);
-    _data.resize(size);
-    auto read = file.read(_data.data(), size);
-    if (read != size) {
-        setErrorString(file.errorString());
-        return false;
-    }
+    _size = qBound(0ll, _size, fileDiskSize - _start);
+    _read = 0;
 
-    return QIODevice::open(QIODevice::ReadOnly);
+    return QIODevice::open(mode);
 }
 
+void UploadDevice::close()
+{
+    _file.close();
+    QIODevice::close();
+}
 
 qint64 UploadDevice::writeData(const char *, qint64)
 {
-    ASSERT(false, "write to read only device");
+    OC_ASSERT_X(false, "write to read only device");
     return 0;
 }
 
 qint64 UploadDevice::readData(char *data, qint64 maxlen)
 {
-    if (_data.size() - _read <= 0) {
+    if (_size - _read <= 0) {
         // at end
         if (_bandwidthManager) {
             _bandwidthManager->unregisterUploadDevice(this);
         }
         return -1;
     }
-    maxlen = qMin(maxlen, _data.size() - _read);
-    if (maxlen == 0) {
+    maxlen = qMin(maxlen, _size - _read);
+    if (maxlen <= 0) {
         return 0;
     }
     if (isChoked()) {
@@ -439,9 +329,14 @@ qint64 UploadDevice::readData(char *data, qint64 maxlen)
         }
         _bandwidthQuota -= maxlen;
     }
-    std::memcpy(data, _data.data() + _read, maxlen);
-    _read += maxlen;
-    return maxlen;
+
+    auto c = _file.read(data, maxlen);
+    if (c < 0) {
+        setErrorString(_file.errorString());
+        return -1;
+    }
+    _read += c;
+    return c;
 }
 
 void UploadDevice::slotJobUploadProgress(qint64 sent, qint64 t)
@@ -449,22 +344,23 @@ void UploadDevice::slotJobUploadProgress(qint64 sent, qint64 t)
     if (sent == 0 || t == 0) {
         return;
     }
+    // used by the BandwidthManager
     _readWithProgress = sent;
 }
 
 bool UploadDevice::atEnd() const
 {
-    return _read >= _data.size();
+    return _read >= _size;
 }
 
 qint64 UploadDevice::size() const
 {
-    return _data.size();
+    return _size;
 }
 
 qint64 UploadDevice::bytesAvailable() const
 {
-    return _data.size() - _read + QIODevice::bytesAvailable();
+    return _size - _read + QIODevice::bytesAvailable();
 }
 
 // random access, we can seek
@@ -478,10 +374,11 @@ bool UploadDevice::seek(qint64 pos)
     if (!QIODevice::seek(pos)) {
         return false;
     }
-    if (pos < 0 || pos > _data.size()) {
+    if (pos < 0 || pos > _size) {
         return false;
     }
     _read = pos;
+    _file.seek(_start + pos);
     return true;
 }
 
@@ -507,36 +404,6 @@ void UploadDevice::setChoked(bool b)
     }
 }
 
-void PropagateUploadFileCommon::startPollJob(const QString &path)
-{
-    PollJob *job = new PollJob(propagator()->account(), path, _item,
-        propagator()->_journal, propagator()->_localDir, this);
-    connect(job, &PollJob::finishedSignal, this, &PropagateUploadFileCommon::slotPollFinished);
-    SyncJournalDb::PollInfo info;
-    info._file = _item->_file;
-    info._url = path;
-    info._modtime = _item->_modtime;
-    propagator()->_journal->setPollInfo(info);
-    propagator()->_journal->commit("add poll info");
-    propagator()->_activeJobList.append(this);
-    job->start();
-}
-
-void PropagateUploadFileCommon::slotPollFinished()
-{
-    PollJob *job = qobject_cast<PollJob *>(sender());
-    ASSERT(job);
-
-    propagator()->_activeJobList.removeOne(this);
-
-    if (job->_item->_status != SyncFileItem::Success) {
-        done(job->_item->_status, job->_item->_errorString);
-        return;
-    }
-
-    finalize();
-}
-
 void PropagateUploadFileCommon::done(SyncFileItem::Status status, const QString &errorString)
 {
     _finished = true;
@@ -559,7 +426,7 @@ void PropagateUploadFileCommon::checkResettingErrors()
                                       << "is" << uploadInfo._errorCount;
         }
         propagator()->_journal->setUploadInfo(_item->_file, uploadInfo);
-        propagator()->_journal->commit("Upload info");
+        propagator()->_journal->commit(QStringLiteral("Upload info"));
     }
 }
 
@@ -574,7 +441,7 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
 
         // Maybe the bad etag is in the database, we need to clear the
         // parent folder etag so we won't read from DB next sync.
-        propagator()->_journal->avoidReadFromDbOnNextSync(_item->_file);
+        propagator()->_journal->schedulePathForRemoteDiscovery(_item->_file);
         propagator()->_anotherSyncNeeded = true;
     }
 
@@ -582,25 +449,22 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
     checkResettingErrors();
 
     SyncFileItem::Status status = classifyError(job->reply()->error(), _item->_httpErrorCode,
-        &propagator()->_anotherSyncNeeded);
+        &propagator()->_anotherSyncNeeded, replyContent);
 
     // Insufficient remote storage.
     if (_item->_httpErrorCode == 507) {
         // Update the quota expectation
-        /* store the quota for the real local file using the information
-         * on the file to upload, that could have been modified by
-         * filters or something. */
         const auto path = QFileInfo(_item->_file).path();
         auto quotaIt = propagator()->_folderQuota.find(path);
         if (quotaIt != propagator()->_folderQuota.end()) {
-            quotaIt.value() = qMin(quotaIt.value(), _fileToUpload._size - 1);
+            quotaIt.value() = qMin(quotaIt.value(), _item->_size - 1);
         } else {
-            propagator()->_folderQuota[path] = _fileToUpload._size - 1;
+            propagator()->_folderQuota[path] = _item->_size - 1;
         }
 
         // Set up the error
         status = SyncFileItem::DetailError;
-        errorString = tr("Upload of %1 exceeds the quota for the folder").arg(Utility::octetsToString(_fileToUpload._size));
+        errorString = tr("Upload of %1 exceeds the quota for the folder").arg(Utility::octetsToString(_item->_size));
         emit propagator()->insufficientRemoteStorage();
     } else if (_item->_httpErrorCode == 405) {
         status = SyncFileItem::FileIgnored;
@@ -619,14 +483,14 @@ void PropagateUploadFileCommon::commonErrorHandling(AbstractNetworkJob *job)
     abortWithError(status, errorString);
 }
 
-void PropagateUploadFileCommon::adjustLastJobTimeout(AbstractNetworkJob *job, quint64 fileSize)
+void PropagateUploadFileCommon::adjustLastJobTimeout(AbstractNetworkJob *job, qint64 fileSize)
 {
     job->setTimeout(qBound(
         job->timeoutMsec(),
         // Calculate 3 minutes for each gigabyte of data
         qint64((3 * 60 * 1000) * fileSize / 1e9),
         // Maximum of 30 minutes
-        qint64(30 * 60 * 1000)));
+        30 * 60 * 1000LL));
 }
 
 void PropagateUploadFileCommon::slotJobDestroyed(QObject *job)
@@ -637,6 +501,8 @@ void PropagateUploadFileCommon::slotJobDestroyed(QObject *job)
 // This function is used whenever there is an error occuring and jobs might be in progress
 void PropagateUploadFileCommon::abortWithError(SyncFileItem::Status status, const QString &error)
 {
+    if (_aborting)
+        return;
     abort(AbortType::Synchronous);
     done(status, error);
 }
@@ -644,11 +510,12 @@ void PropagateUploadFileCommon::abortWithError(SyncFileItem::Status status, cons
 QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
 {
     QMap<QByteArray, QByteArray> headers;
-    headers["OC-Async"] = "1";
-    headers["Content-Type"] = "application/octet-stream";
-    headers["X-OC-Mtime"] = QByteArray::number(qint64(_item->_modtime));
+    headers[QByteArrayLiteral("Content-Type")] = QByteArrayLiteral("application/octet-stream");
+    headers[QByteArrayLiteral("X-OC-Mtime")] = QByteArray::number(qint64(_item->_modtime));
+    if (qEnvironmentVariableIntValue("OWNCLOUD_LAZYOPS"))
+        headers[QByteArrayLiteral("OC-LazyOps")] = QByteArrayLiteral("true");
 
-    if (_item->_file.contains(".sys.admin#recall#")) {
+    if (_item->_file.contains(QLatin1String(".sys.admin#recall#"))) {
         // This is a file recall triggered by the admin.  Note: the
         // recall list file created by the admin and downloaded by the
         // client (.sys.admin#recall#) also falls into this category
@@ -665,19 +532,21 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
         && !_deleteExisting) {
         // We add quotes because the owncloud server always adds quotes around the etag, and
         //  csync_owncloud.c's owncloud_file_id always strips the quotes.
-        headers["If-Match"] = '"' + _item->_etag + '"';
+        headers[QByteArrayLiteral("If-Match")] = '"' + _item->_etag + '"';
     }
 
     // Set up a conflict file header pointing to the original file
     auto conflictRecord = propagator()->_journal->conflictRecord(_item->_file.toUtf8());
     if (conflictRecord.isValid()) {
-        headers["OC-Conflict"] = "1";
+        headers[QByteArrayLiteral("OC-Conflict")] = "1";
+        if (!conflictRecord.initialBasePath.isEmpty())
+            headers[QByteArrayLiteral("OC-ConflictInitialBasePath")] = conflictRecord.initialBasePath;
         if (!conflictRecord.baseFileId.isEmpty())
-            headers["OC-ConflictBaseFileId"] = conflictRecord.baseFileId;
+            headers[QByteArrayLiteral("OC-ConflictBaseFileId")] = conflictRecord.baseFileId;
         if (conflictRecord.baseModtime != -1)
-            headers["OC-ConflictBaseMtime"] = QByteArray::number(conflictRecord.baseModtime);
+            headers[QByteArrayLiteral("OC-ConflictBaseMtime")] = QByteArray::number(conflictRecord.baseModtime);
         if (!conflictRecord.baseEtag.isEmpty())
-            headers["OC-ConflictBaseEtag"] = conflictRecord.baseEtag;
+            headers[QByteArrayLiteral("OC-ConflictBaseEtag")] = conflictRecord.baseEtag;
     }
 
     return headers;
@@ -685,26 +554,50 @@ QMap<QByteArray, QByteArray> PropagateUploadFileCommon::headers()
 
 void PropagateUploadFileCommon::finalize()
 {
+    if (_item->_remotePerm.isNull()) {
+        qCWarning(lcPropagateUpload) << "PropagateUploadFileCommon::finalize: Missing permissions for" << propagator()->fullRemotePath(_item->_file);
+        auto permCheck = new PropfindJob(propagator()->account(), propagator()->fullRemotePath(_item->_file));
+        _jobs.append(permCheck);
+        permCheck->setProperties({ "http://owncloud.org/ns:permissions" });
+        connect(permCheck, &PropfindJob::result, this, [this, permCheck](const QMap<QString, QString> &map) {
+            _item->_remotePerm = RemotePermissions::fromServerString(map.value(QStringLiteral("permissions")));
+            finalize();
+            slotJobDestroyed(permCheck);
+        });
+        connect(permCheck, &QObject::destroyed, this, &PropagateUploadFileCommon::slotJobDestroyed);
+        permCheck->start();
+        return;
+    }
     // Update the quota, if known
     auto quotaIt = propagator()->_folderQuota.find(QFileInfo(_item->_file).path());
     if (quotaIt != propagator()->_folderQuota.end())
-        quotaIt.value() -= _fileToUpload._size;
+        quotaIt.value() -= _item->_size;
 
-    // Update the database entry - use the local file, not the temporary one.
-    const auto filePath = propagator()->getFilePath(_item->_file);
-    const auto fileRecord = _item->toSyncJournalFileRecordWithInode(filePath);
-    if (!propagator()->_journal->setFileRecord(fileRecord)) {
-        done(SyncFileItem::FatalError, tr("Error writing metadata to the database"));
+    // Update the database entry
+    const auto result = propagator()->updateMetadata(*_item);
+    if (!result) {
+        done(SyncFileItem::FatalError, tr("Error updating metadata: %1").arg(result.error()));
         return;
+    } else if (result.get() == Vfs::ConvertToPlaceholderResult::Locked) {
+        done(SyncFileItem::SoftError, tr("The file %1 is currently in use").arg(_item->_file));
+        return;
+    }
+
+    // Files that were new on the remote shouldn't have online-only pin state
+    // even if their parent folder is online-only.
+    if (_item->_instruction == CSYNC_INSTRUCTION_NEW
+        || _item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
+        auto &vfs = propagator()->syncOptions()._vfs;
+        const auto pin = vfs->pinState(_item->_file);
+        if (pin && *pin == PinState::OnlineOnly) {
+            vfs->setPinState(_item->_file, PinState::Unspecified);
+        }
     }
 
     // Remove from the progress database:
     propagator()->_journal->setUploadInfo(_item->_file, SyncJournalDb::UploadInfo());
-    propagator()->_journal->commit("upload file start");
+    propagator()->_journal->commit(QStringLiteral("upload file start"));
 
-    if (_uploadingEncrypted) {
-      _uploadEncryptedHelper->unlockFolder();
-    }
     done(SyncFileItem::Success);
 }
 
@@ -712,6 +605,10 @@ void PropagateUploadFileCommon::abortNetworkJobs(
     PropagatorJob::AbortType abortType,
     const std::function<bool(AbstractNetworkJob *)> &mayAbortJob)
 {
+    if (_aborting)
+        return;
+    _aborting = true;
+
     // Count the number of jobs that need aborting, and emit the overall
     // abort signal when they're all done.
     QSharedPointer<int> runningCount(new int(0));
@@ -723,7 +620,7 @@ void PropagateUploadFileCommon::abortNetworkJobs(
     };
 
     // Abort all running jobs, except for explicitly excluded ones
-    foreach (AbstractNetworkJob *job, _jobs) {
+    for (auto *job : qAsConst(_jobs)) {
         auto reply = job->reply();
         if (!reply || !reply->isRunning())
             continue;
